import { ɵɵdefineInjectable, Injectable, Component, ChangeDetectorRef, Input, HostListener, Pipe, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { FormsModule } from '@angular/forms';

class SearchableJsonViewerService {
    constructor() { }
}
SearchableJsonViewerService.ɵprov = ɵɵdefineInjectable({ factory: function SearchableJsonViewerService_Factory() { return new SearchableJsonViewerService(); }, token: SearchableJsonViewerService, providedIn: "root" });
SearchableJsonViewerService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
SearchableJsonViewerService.ctorParameters = () => [];

class SearchableJsonViewerComponent {
    constructor(cdr) {
        this.cdr = cdr;
        this.expandedNodes = new Set();
        this.searchTerm = '';
        this.navigateId = 0;
        this.totalHighlightsSize = 0;
        this.makeExpandable = (data) => {
            try {
                if (typeof data === "object" && !Array.isArray(data)) {
                    // If it's an object, add it to the Set
                    this.expandedNodes.add(data);
                    // Recursively traverse the object's properties
                    for (const key in data) {
                        this.makeExpandable(data[key]);
                    }
                }
                else if (Array.isArray(data)) {
                    // If it's an array, add it to the Set
                    this.expandedNodes.add(data);
                    // Recursively traverse the array's elements
                    data.forEach(item => {
                        this.makeExpandable(item);
                    });
                }
            }
            catch (ex) {
                console.log("Error in makeExpandable::" + ex);
            }
        };
        this.isCheckCondition = (value) => {
            if (this.isObject(value) || this.isArray(value)) {
                return this.isNodeExpanded(value);
            }
            else {
                return true;
            }
        };
        this.next = () => {
            if (this.searchTerm.length > 0) {
                const highlightedSpans = document.querySelectorAll("span.highlight");
                if (Number(this.navigateId) < highlightedSpans.length) {
                    this.navigateId++;
                    highlightedSpans.forEach((span) => {
                        if (span) {
                            const spanId = span.getAttribute("id");
                            if (Number(spanId) === Number(this.navigateId)) {
                                span.classList.add("navigator-bg");
                                span.scrollIntoView({ behavior: 'smooth', block: 'center' });
                            }
                            else {
                                span.classList.remove("navigator-bg");
                            }
                        }
                    });
                }
                //Commented for temeraory which is used to loop purpose
                // if(highlightedSpans.length < Number(this.navigateId)){
                //   this.navigateId = 0;
                //   this.next();
                // }
            }
        };
        this.prev = () => {
            if (this.searchTerm.length > 0) {
                if (this.navigateId > 1) {
                    this.navigateId--;
                    const highlightedSpans = document.querySelectorAll("span.highlight");
                    highlightedSpans.forEach((span) => {
                        if (span) {
                            const spanId = span.getAttribute("id");
                            if (Number(spanId) === Number(this.navigateId)) {
                                span.classList.add("navigator-bg");
                                span.scrollIntoView({ behavior: 'smooth', block: 'center' });
                            }
                            else {
                                span.classList.remove("navigator-bg");
                            }
                        }
                    });
                }
            }
        };
        this.getTotalHighlights = () => {
            const highlightedSpans = document.querySelectorAll("span.highlight");
            let size = highlightedSpans.length;
            return size;
        };
    }
    handleKeyPress(event) {
        if (event.ctrlKey && event.altKey && event.which == 70) {
            let txt = window.getSelection();
            let searchTerm = txt.toString();
            this.searchJson(searchTerm);
        }
    }
    ngOnInit() {
        this.makeExpandable(this.json);
    }
    ngAfterContentChecked() {
        this.totalHighlightsSize = this.getTotalHighlights();
        this.cdr.detectChanges();
    }
    isObject(value) {
        let type = typeof value === 'object' && value !== null && !Array.isArray(value);
        return type;
    }
    isArray(value) {
        return Array.isArray(value);
    }
    getObjectKeys(obj) {
        let keys = Object.keys(obj);
        return keys;
    }
    toggleNode(node) {
        if (this.expandedNodes.has(node)) {
            this.expandedNodes.delete(node);
        }
        else {
            this.expandedNodes.add(node);
            //open all the element from the heirarichy
            this.makeExpandable(node);
        }
        setTimeout(() => {
            if (this.searchTerm.length > 0) {
                const highlightedSpans = document.querySelectorAll("span.highlight");
                highlightedSpans.forEach((span) => {
                    span.parentNode.replaceChild(document.createTextNode(span.textContent), span);
                });
                this.searchJson(this.searchTerm);
            }
        }, 10);
    }
    isNodeExpanded(node) {
        return this.expandedNodes.has(node);
    }
    searchJson(searchText) {
        const spanTexts = document.querySelectorAll(".segment-value");
        if (searchText !== this.searchTerm) { //implemented when typing
            const highlightedSpans = document.querySelectorAll("span.highlight");
            highlightedSpans.forEach((span) => {
                span.parentNode.replaceChild(document.createTextNode(span.textContent), span);
            });
            //clear all the items
            if (searchText === '') {
                this.searchTerm = searchText;
                return;
            }
        }
        let highlightedSpanTexts = [];
        for (let spanElement of spanTexts) {
            const text = spanElement.innerHTML;
            if (text.toLowerCase().indexOf(searchText.toLowerCase()) > -1) {
                highlightedSpanTexts.push(spanElement);
            }
        }
        let index = 0;
        for (let content of highlightedSpanTexts) {
            const text = content.innerHTML;
            const highlightedText = text.replace(new RegExp(searchText, 'gi'), (match) => `<span id=${index + 1} class="highlight">${match}</span>`);
            content.innerHTML = highlightedText;
            index++;
        }
        this.searchTerm = searchText;
    }
}
SearchableJsonViewerComponent.decorators = [
    { type: Component, args: [{
                selector: 'searchable-json-viewer',
                template: `
    <div
    class="header">
      <input 
      class="input-custom"
      type="text" 
      [ngModel]="searchTerm" 
      (ngModelChange)="searchJson($event)" 
      placeholder="Search JSON">
      <div class="action-container">
        <div class="segment-key" *ngIf="totalHighlightsSize > 0">
          {{navigateId + '/' + totalHighlightsSize}}
        </div>
        <button class="btn prev-button" (click)="prev()">&#x2039;</button>
        <button class="btn next-button" (click)="next()">&#x203A;</button>
      </div>
    </div>
    
    <div class="json-viewer-container">
      <ng-container 
      [ngTemplateOutlet]="renderNode" 
      [ngTemplateOutletContext]="{ 
        $implicit: {
          node: json
        } 
      }"></ng-container>
    </div>
    <ng-template #renderNode let-data>
      <div 
      class="section-node"
      style="margin-left: 24px;"
      *ngIf="isObject(data.node)">
        <div *ngFor="let key of getObjectKeys(data.node)">
          <span 
          class="arrow"
          *ngIf="isObject(data.node[key]) || isArray(data.node[key])"
          [ngClass]="{'down-arrow': isNodeExpanded(data.node[key]), 'right-arrow': !isNodeExpanded(data.node[key])}"
          (click)="toggleNode(data.node[key])">&#x27A4;</span>
          <strong class="segment-key">{{ key }}: </strong>
          <ng-container 
          *ngIf="isCheckCondition(data.node[key])"
          [ngTemplateOutlet]="renderNode"
          [ngTemplateOutletContext]="{ 
            $implicit: {
              node: data.node[key]
            } 
          }"></ng-container>
        </div>
      </div>
      <div *ngIf="isArray(data.node)">
        <div 
        style="margin-left: 12px;"
        *ngFor="let item of data.node; let i = index">
          <span 
          class="arrow"
          *ngIf="isObject(item) || isArray(item)"
          [ngClass]="{'down-arrow': isNodeExpanded(item), 'right-arrow': !isNodeExpanded(item)}"
          (click)="toggleNode(item)">&#x27A4;</span>
          <strong class="segment-key">{{ i + ': ' }}</strong>
          <!--following span only applicable for array of string or array of intergers Begin-->
          <span 
          *ngIf="!isObject(item) && !isArray(item)" 
          class="segment-value">
            {{ item | jsonValue }}
          </span>
          <!--End-->
          <ng-container 
          *ngIf="isNodeExpanded(item)"
          [ngTemplateOutlet]="renderNode"
          [ngTemplateOutletContext]="{ 
            $implicit: {
              node: item
            } 
          }"></ng-container>
        </div>
      </div>
      <span
      *ngIf="!isObject(data.node) && !isArray(data.node)" 
      class="segment-value">
        {{ data.node | jsonValue }}
      </span>
    </ng-template>  
  `,
                styles: [".json-viewer-container{margin:10px 0 0}.json-viewer-container .segment-key{color:#6c5e5e;font-family:Arial,Helvetica,sans-serif;font-size:12px}.json-viewer-container .segment-value{color:#f78c6c;font-family:Arial,Helvetica,sans-serif;font-size:12px}.json-viewer-container ::ng-deep .highlight{background:#ff0}.json-viewer-container ::ng-deep .navigator-bg{background:#fdbb04!important}.json-viewer-container .section-node:first-child{margin-left:0!important}.json-viewer-container .arrow{cursor:pointer;margin-right:5px;font-size:12px;color:#6c5e5e;-moz-user-select:none;-webkit-user-select:none;user-select:none}.json-viewer-container .down-arrow{transform:rotate(90deg);display:inline-block}.json-viewer-container .right-arrow{transform:none}.header{position:sticky;top:0;background:#fff;padding:10px;z-index:2;display:flex;border-radius:2px;border:1px solid #dcdcdc;gap:5px;align-items:center;justify-content:space-between}.header .action-container{display:flex;align-items:center;gap:10px}.header .action-container .btn{background:transparent;border:none;cursor:pointer;color:#6c5e5e;font-size:20px}.header .action-container .btn:active{transform:scale(.98);box-shadow:3px 2px 22px 1px rgba(0,0,0,.24)}.header .input-custom{outline:none;border:none;width:50%}.header .segment-key{color:#6c5e5e;font-family:Arial,Helvetica,sans-serif;font-size:12px}"]
            },] }
];
SearchableJsonViewerComponent.ctorParameters = () => [
    { type: ChangeDetectorRef }
];
SearchableJsonViewerComponent.propDecorators = {
    json: [{ type: Input }],
    handleKeyPress: [{ type: HostListener, args: ['document:keydown', ['$event'],] }]
};

class JsonValuePipe {
    transform(value) {
        return isNaN(value) ? JSON.stringify(value) : value;
    }
}
JsonValuePipe.decorators = [
    { type: Pipe, args: [{
                name: 'jsonValue'
            },] }
];

class SearchableJsonViewerModule {
}
SearchableJsonViewerModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    SearchableJsonViewerComponent,
                    JsonValuePipe
                ],
                imports: [
                    CommonModule,
                    FormsModule
                ],
                exports: [
                    SearchableJsonViewerComponent
                ]
            },] }
];

/*
 * Public API Surface of searchable-json-viewer
 */

/**
 * Generated bundle index. Do not edit.
 */

export { SearchableJsonViewerComponent, SearchableJsonViewerModule, SearchableJsonViewerService, JsonValuePipe as ɵa };
//# sourceMappingURL=searchable-json-viewer.js.map
